@@ -6,19 +6,44 @@ from neuralflow.gpu import *
 from neuralflow.utils import *
 
 
-class BackBone:
+class BaseModel:
     def __init__(self):
         self.network = OrderedDict()
 
     def __call__(self, arg):
-        result = self._forward(arg)
+        result = self.forward(arg)
         return result
 
-    def _forward():
+    def forward():
         pass
 
 
-class DenseLayer():
+class BaseLayer():
+    def __init__(self):
+        self.differentiable = False
+        self.changeability = False
+        self.tying = False
+        self.tied = False
+        self.parameter = OrderedDict()
+
+    def _to_cpu(self):
+        param_list = list(self.parameter.keys())
+        for param in param_list:
+            self.parameter[param] = to_cpu(self.parameter[param])
+
+    def _to_gpu(self):
+        param_list = list(self.parameter.keys())
+        for param in param_list:
+            self.parameter[param] = to_gpu(self.parameter[param])
+
+    def _save_params(self):
+        return deepcopy(self.parameter)
+
+    def _load_params(self, params):
+        self.parameter = deepcopy(params)
+
+
+class DenseLayer(BaseLayer):
     def __init__(self, input_size: int, output_size: int, initialize: str = "He"):
         """
         Initialize DenseLayer
@@ -32,16 +57,13 @@ class DenseLayer():
         initialize (str, optional) : 가중치 초기화 방법 설정. Default: "He"
 
         """
+        super().__init__()
         self.input_size = input_size
         self.output_size = output_size
-        self.changeability = False
         self.differentiable = True
-        self.tying = False
-        self.tied = False
         self.x = None
         # for 4-dim tensor
         self.orgin_x_shape = None
-        self.parameter = OrderedDict()
 
         if initialize == "He":
             self.parameter["weight"] = np.random.randn(input_size, output_size).astype(np.float32) * (
@@ -145,7 +167,7 @@ class DenseLayer():
         return grad
 
 
-class Embedding():
+class Embedding(BaseLayer):
     def __init__(self, parameter):
         """
         Initialize EmbeddingLayer
@@ -159,10 +181,9 @@ class Embedding():
         initialize (str, optional) : 가중치 초기화 방법 설정. Default: "He"
 
         """
+        super().__init__()
         self.differentiable = True
-        self.changeability = False
         self.index = None
-        self.parameter = OrderedDict()
         self.parameter["weight"] = parameter["weight"]
         self.dw = np.zeros_like(self.parameter["weight"])
 
@@ -209,14 +230,12 @@ class Embedding():
         return dw
 
 
-class EmbeddingLayer():
+class EmbeddingLayer(BaseLayer):
     def __init__(self, vocab_size: int, hidden_size: int, initialize="He"):
+        super().__init__()
         self.vocab_size = vocab_size
         self.hidden_size = hidden_size
-        self.changeability = False
-        self.tied = False
         self.differentiable = True
-        self.parameter = OrderedDict()
 
         if initialize == "He":
             self.parameter["weight"] = np.random.randn(vocab_size, hidden_size).astype(np.float32) * (
@@ -279,7 +298,7 @@ class EmbeddingLayer():
         return grad
 
 
-class ConvLayer():
+class ConvLayer(BaseLayer):
     """
     Convolution layer
     """
@@ -304,10 +323,8 @@ class ConvLayer():
 
 
         """
+        super().__init__()
         self.differentiable = True
-        self.tied = False
-        self.changeability = False
-        self.parameter = OrderedDict()
         self.input_channel = input_channel
         self.output_channel = output_channel
 
@@ -348,7 +365,7 @@ class ConvLayer():
         else:
             raise ValueError("'initialize' must be 'He' or 'Xavier' or 'None'")
 
-    def __repr__(self) -> str:
+    def __repr__(self):
         return "ConvLayer"
 
     def __call__(self, arg):
@@ -428,11 +445,9 @@ class ConvLayer():
         return img[:, :, self.padding:input_height + self.padding, self.padding:input_width + self.padding]
 
 
-class MaxPoolingLayer():
+class MaxPoolingLayer(BaseLayer):
     def __init__(self, kernel_size, stride=1, padding=0):
-        self.differentiable = False
-        self.tied = False
-        self.changeability = False
+        super().__init__()
 
         if isinstance(kernel_size, int):
             self.kernel_width = kernel_size
@@ -447,7 +462,7 @@ class MaxPoolingLayer():
         self.x = None
         self.mask = None
 
-    def __repr__(self) -> str:
+    def __repr__(self):
         return "MaxPoolingLayer"
 
     def __call__(self, arg):
@@ -521,11 +536,10 @@ class MaxPoolingLayer():
         return img[:, :, self.padding:input_height + self.padding, self.padding:input_width + self.padding]
 
 
-class RNNCell():
+class RNNCell(BaseLayer):
     def __init__(self, parameter):
-        self.parameter = OrderedDict()
+        super().__init__()
         self.differentiable = True
-        self.changeability = False
         self.parameter["weight_x"] = parameter["weight_x"]
         self.parameter["weight_h"] = parameter["weight_h"]
         self.parameter["bias"] = parameter["bias"]
@@ -577,12 +591,10 @@ class RNNCell():
         return dx, dwx, dwh, db
 
 
-class RNNLayer():
+class RNNLayer(BaseLayer):
     def __init__(self, input_size, hidden_size, n_layers=1, bidirectional=True, initialize="He"):
+        super().__init__()
         self.differentiable = True
-        self.tied = False
-        self.changeability = False
-        self.parameter = OrderedDict()
         self.input_size = input_size
         self.hidden_size = hidden_size
 
@@ -688,11 +700,10 @@ class RNNLayer():
         self.h = None
 
 
-class LSTMCell():
+class LSTMCell(BaseLayer):
     def __init__(self, parameter):
+        super().__init__()
         self.differentiable = True
-        self.changeability = False
-        self.parameter = OrderedDict()
         self.parameter["weight_x"] = parameter["weight_x"]
         self.parameter["weight_h"] = parameter["weight_h"]
         self.parameter["bias"] = parameter["bias"]
@@ -702,8 +713,8 @@ class LSTMCell():
         self.dwh = None
         self.db = None
 
-    def __call__(self, *arg):
-        result = self._forward(*arg)
+    def __call__(self, *args):
+        result = self._forward(*args)
         return result
 
     def _forward(self, x, h_t_prev, c_t_prev):
@@ -768,12 +779,10 @@ class LSTMCell():
         return dx, dwx, dwh, db
 
 
-class LSTMLayer():
+class LSTMLayer(BaseLayer):
     def __init__(self, input_size, hidden_size, n_layers=1, bidirectional=True, initialize="He"):
+        super().__init__()
         self.differentiable = True
-        self.tied = False
-        self.changeability = False
-        self.parameter = OrderedDict()
         self.input_size = input_size
         self.hidden_size = hidden_size
 
@@ -800,7 +809,7 @@ class LSTMLayer():
         self.c = None
         self.dh = None
         self.layer = None
-        self.stateful = "stateful"
+        self.stateful = True
         self.dx = None
         self.dwx = None
         self.dwh = None
@@ -880,10 +889,9 @@ class LSTMLayer():
         self.h, self.c = None, None
 
 
-class Dropout():
+class Dropout(BaseLayer):
     def __init__(self, dropout_ratio=0.5):
-        self.differentiable = False
-        self.tied = False
+        super().__init__()
         self.changeability = True
         self.dropout_ratio = dropout_ratio
         self.train_flg = True
@@ -913,27 +921,12 @@ class Dropout():
         self.train_flg = False
 
 
-class BatchNorm():
-    def __init__(self, epsilon=1e-8):
-        self.differentiable = True
-        self.mean = None
-        self.std = None
-        self.parameter = OrderedDict()
-
-    def _forward(self, x):
-        self.x = x
-        result = np.matmul(x, self.parameter["weight"]) + self.parameter["bias"]
-        return result
-
-    def __repr__(self):
-        return "BatchNormLayer"
-
-
-class Model(BackBone):
+class Model(BaseModel):
     def __init__(self, *layers):
         super().__init__()
         self.sequence = []
         self.grad = OrderedDict()
+
         self.layers = layers
 
         self.count_dict = OrderedDict()
@@ -1017,6 +1010,7 @@ class Model(BackBone):
         if repr(layer) not in self.count_dict.keys():
             self.count_dict[repr(layer)] = 1
 
+        self.layers += (layer,)
         self.network[f"{repr(layer)}{self.count_dict[repr(layer)]}"] = layer
         self.sequence.append(f"{repr(layer)}{self.count_dict[repr(layer)]}")
         self.count_dict[repr(layer)] += 1
@@ -1052,16 +1046,37 @@ class Model(BackBone):
         emb_layer.parameter["weight"] = final_dense_layer.parameter["weight"].T
         final_dense_layer.emb_dw = emb_layer.dw
 
-    def save_model(self, fn=None):
-        if file_name is None:
-            file_name = self.__class__.__name__ + '.pkl'
+    def to_cpu(self):
+        for layer_name in self.sequence:
+            layer = self.network[layer_name]
+            if layer.differentiable:
+                layer._to_cpu()
 
-        params = [p.astype(np.float16) for p in self.network]
-        if GPU:
-            params = [to_cpu(p) for p in params]
+    def to_gpu(self):
+        for layer_name in self.sequence:
+            layer = self.network[layer_name]
+            if layer.differentiable:
+                layer._to_gpu()
 
-        with open(file_name, 'wb') as f:
-            pickle.dump(params, f)
+    def save_params(self, fn=None):
+        model_param = OrderedDict()
+        for layer_name in self.sequence:
+            layer = self.network[layer_name]
+            if layer.differentiable:
+                param = layer._save_params()
+                model_param[layer_name] = param
+
+        if fn is None:
+            fn = self.__class__.__name__ + '.pkl'
+
+        with open(fn, 'wb') as f:
+            pickle.dump(model_param, f)
 
     def load_params(self, fn=None):
-        pass
\ No newline at end of file
+        with open(fn, 'rb') as f:
+            model_param = deepcopy(pickle.load(f))
+
+        for layer_name in self.sequence:
+            layer = self.network[layer_name]
+            if layer.differentiable:
+                layer._load_params(model_param[layer_name])
