@@ -1,7 +1,5 @@
-import sys
 from tqdm.notebook import tqdm
 import matplotlib.pyplot as plt
-import plotly.express as px
 from collections import OrderedDict
 from neuralflow.gpu import *
 from neuralflow.utils import *
@@ -79,6 +77,7 @@ class ClassificationTrainer(BaseTrainer):
 
         """
         for epoch in range(self.n_epochs):
+            self.model.train_state()
             print(f"epoch {epoch + 1}")
             tmp_train_loss = np.array([])
             train_correct_num = 0
@@ -93,13 +92,16 @@ class ClassificationTrainer(BaseTrainer):
                 train_correct_num += np.sum(pred == y)
                 y_num += len(y)
 
-                print(f"train loss : {train_loss}    train accuarcy : {train_correct_num / y_num * 100}\r", end="")
+                train_temp_accuracy = train_correct_num / y_num * 100
+
+                print(f"train loss : {train_loss:.6f}    train accuarcy : {train_temp_accuracy:.6f}\r", end="")
 
                 self._backward()
                 self._update()
 
             epoch_train_loss = np.sum(tmp_train_loss) / train_dataloader.batch_size
             self.train_loss_list = np.append(self.train_loss_list, epoch_train_loss)
+
             dataset_len = train_dataloader.dataset_len()
             epoch_train_accuracy = train_correct_num / float(dataset_len) * 100
             self.train_accuracy_list = np.append(self.train_accuracy_list, epoch_train_accuracy)
@@ -109,9 +111,11 @@ class ClassificationTrainer(BaseTrainer):
             if valid_dataloader is not None:
                 self._validate(valid_dataloader, epoch)
             else:
-                print("--------------------------------")
+                print("----------------------------------------------------------------")
 
     def _validate(self, valid_dataloader, epoch):
+        self.model.valid_state()
+        self.model.reset_rnn_state()
         tmp_valid_loss = np.array([])
         valid_correct_num = 0
         y_num = 0
@@ -124,16 +128,20 @@ class ClassificationTrainer(BaseTrainer):
             valid_correct_num += np.sum(pred == y)
             y_num += len(y)
 
-            print(f"valid loss : {valid_loss}    valid accuarcy : {valid_correct_num / y_num * 100}\r", end="")
+            valid_temp_accuracy = valid_correct_num / y_num * 100
+
+            print(f"valid loss : {valid_loss:.6f}    valid accuarcy : {valid_temp_accuracy:.6f}\r", end="")
 
         epoch_valid_loss = np.sum(tmp_valid_loss) / valid_dataloader.batch_size
         self.valid_loss_list = np.append(self.valid_loss_list, epoch_valid_loss)
         dataset_len = valid_dataloader.dataset_len()
         epoch_valid_accuracy = valid_correct_num / float(dataset_len) * 100
         self.valid_accuracy_list = np.append(self.valid_accuracy_list, epoch_valid_accuracy)
+        self.model.reset_rnn_state()
+
         print()
         print(f"epoch {epoch + 1} -- valid loss : {epoch_valid_loss}    valid accuarcy : {epoch_valid_accuracy}")
-        print("--------------------------------")
+        print("----------------------------------------------------------------")
 
     def show_error_graph(self, valid=False):
         """
@@ -213,7 +221,7 @@ class LanguageModelTrainer(BaseTrainer):
     def _update(self):
         self.optimizer.update(self.model)
 
-    def train(self, train_dataloader, valid_dataloader=None, max_grad=None, iter_num=20):
+    def train(self, train_dataloader, valid_dataloader=None, max_grad=None, show_iter_num=20):
         """
         Train model with train/valid data
 
@@ -224,8 +232,9 @@ class LanguageModelTrainer(BaseTrainer):
         valid_dataloader (iterable) : Valid dataloader that can iterate with batch size in valid dataset
 
         """
-        self.model.train_state()
+
         for epoch in range(self.n_epochs):
+            self.model.train_state()
             print(f"epoch {epoch + 1}")
             tmp_train_loss = np.array([])
             tmp_train_perplexity = np.array([])
@@ -243,13 +252,13 @@ class LanguageModelTrainer(BaseTrainer):
                 tmp_train_perplexity = np.append(tmp_train_perplexity, train_perplexity)
                 iter_tmp_train_perplexity = np.append(iter_tmp_train_perplexity, train_perplexity)
 
-                if count % iter_num == 0:
-                    iter_train_perplexity = np.sum(iter_tmp_train_perplexity) / iter_num
+                if count % show_iter_num == 0:
+                    iter_train_perplexity = np.sum(iter_tmp_train_perplexity) / show_iter_num
                     iter_tmp_train_perplexity = np.array([])
                     self.train_perplexity_list_iter = np.append(self.train_perplexity_list_iter, iter_train_perplexity)
 
                 print(
-                    f"train loss : {train_loss}    train perplexity : {train_perplexity}    iter : {count}/{iter_num}\r",
+                    f"train loss : {train_loss:.6f}    train perplexity : {train_perplexity:.6f}    iter : {count}/{iter_num}\r",
                     end="")
 
                 self._backward()
@@ -265,15 +274,15 @@ class LanguageModelTrainer(BaseTrainer):
 
             print()
             print(
-                f"epoch {epoch + 1} -- train loss : {epoch_train_loss}    train perplexity : {epoch_train_perplexity}")
+                f"epoch {epoch + 1} -- train loss : {epoch_train_loss:.6f}    train perplexity : {epoch_train_perplexity:.6f}")
 
             if valid_dataloader is not None:
                 self._validate(valid_dataloader, epoch)
             else:
-                print("--------------------------------")
+                print("----------------------------------------------------------------")
                 print()
 
-    def _validate(self, valid_dataloader, epoch, iter_num=20):
+    def _validate(self, valid_dataloader, epoch, show_iter_num=20):
         self.model.valid_state()
         self.model.reset_rnn_state()
         tmp_valid_loss = np.array([])
@@ -290,12 +299,12 @@ class LanguageModelTrainer(BaseTrainer):
             tmp_valid_perplexity = np.append(tmp_valid_perplexity, valid_perplexity)
             iter_tmp_valid_perplexity = np.append(iter_tmp_valid_perplexity, valid_perplexity)
 
-            if count % iter_num == 0:
-                iter_valid_perplexity = np.sum(iter_tmp_valid_perplexity) / iter_num
+            if count % show_iter_num == 0:
+                iter_valid_perplexity = np.sum(iter_tmp_valid_perplexity) / show_iter_num
                 iter_tmp_valid_perplexity = np.array([])
                 self.valid_perplexity_list_iter = np.append(self.valid_perplexity_list_iter, iter_valid_perplexity)
 
-            print(f"valid loss : {valid_loss}    valid perplexity : {valid_perplexity}\r", end="")
+            print(f"valid loss : {valid_loss:.6f}    valid perplexity : {valid_perplexity:.6f}\r", end="")
 
         epoch_valid_loss = np.sum(tmp_valid_loss) / count
         self.valid_loss_list = np.append(self.valid_loss_list, epoch_valid_loss)
@@ -305,8 +314,9 @@ class LanguageModelTrainer(BaseTrainer):
         self.model.reset_rnn_state()
 
         print()
-        print(f"epoch {epoch + 1} -- valid loss : {epoch_valid_loss}    valid perplexity : {epoch_valid_perplexity}")
-        print("--------------------------------")
+        print(
+            f"epoch {epoch + 1} -- valid loss : {epoch_valid_loss:.6f}    valid perplexity : {epoch_valid_perplexity:.6f}")
+        print("----------------------------------------------------------------")
         print()
 
     def clip_grad(self, max_norm):
@@ -386,6 +396,26 @@ class LanguageModelTrainer(BaseTrainer):
             plt.grid()
             plt.show()
 
-    def add_metric(self, **metric):
-        print(f"metic {metric.keys()} added")
-        self.metric = dict(self.metric, metric)
\ No newline at end of file
+    def eval_perplexity(self, test_dataloader):
+        print('Test perplexity ...')
+        self.model.valid_state()
+        self.model.reset_rnn_state()
+
+        tmp_test_loss = np.array([])
+        tmp_test_perplexity = np.array([])
+
+        count = 0
+        for x, y in tqdm(test_dataloader):
+            count += 1
+            # test loss
+            test_loss = self._forward(x, y)
+            tmp_test_loss = np.append(tmp_test_loss, test_loss)
+            # test perplexity
+            test_perplexity = np.exp(test_loss)
+            tmp_test_perplexity = np.append(tmp_test_perplexity, test_perplexity)
+
+        epoch_test_loss = np.sum(tmp_test_loss) / count
+        epoch_test_perplexity = np.sum(tmp_test_perplexity) / count
+
+        print()
+        print(f"test loss : {epoch_test_loss:.6f}    test perplexity : {epoch_test_perplexity:.6f}")
\ No newline at end of file
